//#ifndef _HPPREFS_H
//#define _HPPREFS_H
// We can't guard this header as we make multiple passes thru it.

#if defined(DECLARE_PREFS)
	bool		dither;
	bool		random;
	int		attenuation;
	bool		rxdcb;
	bool		txdcb;
	bool		pwsmode;

	bool		anf;
	bool		iqenable;
	bool		sdrom;
	bool		spectrumpolyphase;
	bool		squelchstate;
	bool		subrxquelchstate;
	bool		testbutton;
	int		subrx_gain;
	int		testslider;

	int		spnr_hermes;
	bool		btn_hermes;

	string		filter_low;
	string		filter_high;

	bool		spectrum;
	string		spectrum_fps;
	string		spectrum_samples;

	bool		audio;
	string		audio_buf_size;
	int		audio_rate;
	int		audio_channels;
	int		audio_mic_encode;

	int		rx_encoding;
	int		rx_agc;
	string		fixed_agc;

	int		m160_freq;
	int		m160_mode;
	int		m80_freq;
	int		m80_mode;
	int		m40_freq;
	int		m40_mode;
	int		m30_freq;
	int		m30_mode;
	int		m20_freq;
	int		m20_mode;
	int		m17_freq;
	int		m17_mode;
	int		m15_freq;
	int		m15_mode;
	int		m12_freq;
	int		m12_mode;
	int		m10_freq;
	int		m10_mode;
	int		m6_freq;
	int		m6_mode;
	int		mgen_freq;
	int		mgen_mode;

	double		ppm_hos;
	int		hz_hos;

	string		hxmit_os;
	string		hrcv_os;
	string		hdial_freq;

	// NEW_WIDGET TYPE NAME ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

#undef DECLARE_PREFS
#endif // DECLARE_PREFS

#if defined(INIT_PREFS)
// #include "HPDSP.h"
#define FM_	5
#define AM_	6
#define DIGU_	7

	true,		// int dither;
	true,		// int random;
	0,		// int attenuation;
	false,		// int rxdcb;
	false,		// int txdcb;
	false,		// bool pwsmode;

	false,		// bool anf;
	false,		// bool iqenable;
	false,		// bool sdrom;
	false,		// bool spectrumpolyphase;
	false,		// bool squelchstate;
	false,		// bool subrxquelchstate;
	false,		// bool testbutton;
	0,		// subrx_gain
	0,		// int testslider;

	1,		// int spnr_hermes;
	false,		// bool	btn_hermes;

	"0",		// string filter_low;
	"3000",		// string filter_high;

	false,		// bool	spectrum;
	"15",		// string spectrum_fps;
	"960",		// string spectrum_samples;

	false,		// bool	audio;
	"2048",		// string audio_buf_size;
	0,		// int audio_rate;
	0,		// int audio_channels;
	0,		// int audio_mic_encode;

	1,		// int rx_encoding;
	0,		// int rx_agc;
	"0",		// string fixed_agc;

	1807000,	// int m160_freq;
	DIGU_,		// int m160_mode;
	3580000,	// int m80_freq;
	DIGU_,		// int m80_mode;
	7070000,	// int m40_freq;
	DIGU_,		// int m40_mode;
	10142000,	// int m30_freq;
	DIGU_,		// int m30_mode;
	14070000,	// int m20_freq;
	DIGU_,		// int m20_mode;
	18100000,	// int m17_freq;
	DIGU_,		// int m17_mode;
	21070000,	// int m15_freq;
	DIGU_,		// int m15_mode;
	24920000,	// int m12_freq;
	DIGU_,		// int m12_mode;
	28120000,	// int m10_freq;
	DIGU_,		// int m10_mode;
	52000000,	// int m6_freq;
	FM_,		// int m6_mode;
	850000,		// int mgen_freq;
	AM_,		// int mgen_mode;

	0.0,		// double ppm_hos; XXX double
	0,		// int hz_hos;

	"0",		// string hxmit_os;
	"0",		// string hrcv_os;
	"14070000",	// string hdial_freq;

	// NEW_WIDGET DEFAULT_VALUE // COMMENT ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

#undef INIT_PREFS
#endif // INIT_PREFS

#if defined(OVERRIDE_UI_PREFS)
	255,		// int	bg_red;
	253,		// int	bg_green;
	222,		// int	bg_blue;

	0,		// int	fg_red;
	0,		// int	fg_green;
	0,		// int	fg_blue;

	148,		// int	swrRed;
	0,		// int	swrGreen;
	148,		// int	swrBlue;

	180,		// int	pwrRed;
	0,		// int	pwrGreen;
	0,		// int	pwrBlue;

	0,		// int	smeterRed;
	180,		// int	smeterGreen;
	0,		// int	smeterBlue;

	255,		// int	peakRed;
	0,		// int	peakGreen;
	0,		// int	peakBlue;

	0,		// int	fg_sys_red;
	0,		// int	fg_sys_green;
	0,		// int	fg_sys_blue;

	0xc0,		// int	bg_sys_red;
	0xc0,		// int	bg_sys_green;
	0xc0,		// int	bg_sys_blue;

	255,		// int	bg2_sys_red;
	255,		// int	bg2_sys_green;
	255,		// int	bg2_sys_blue;

	255,		// int	slider_red;
	242,		// int	slider_green;
	190,		// int	slider_blue;

	125,		// int	slider_btn_red;
	100,		// int	slider_btn_green;
	65,		// int	slider_btn_blue;

	255,		// int	lighted_btn_red;
	255,		// int	lighted_btn_green;
	0,		// int	lighted_btn_blue;

	FL_COURIER,	// Fl_Font fontnbr;

	false,		// bool	 tooltips;

	"plastic",	// string ui_scheme

#undef OVERRIDE_UI_PREFS
#endif // OVERRIDE_UI_PREFS

//#endif // _HPPREFS_H
