// =====================================================================
// pixmaps.h
// ----------------------------------------------------------------------------
// Copyright (C) 2014
//              David Freese, W1HKJ
//
// This file is part of flrig.
//
// flrig is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// flrig is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------

#ifndef PIXMAPS_H_
#define PIXMAPS_H_

extern const char *address_book_icon[];
extern const char *edit_undo_icon[];
extern const char *edit_select_all_icon[];
extern const char *edit_clear_icon[];
extern const char *edit_copy_icon[];
extern const char *edit_cut_icon[];
extern const char *edit_paste_icon[];
extern const char *file_open_icon[];
extern const char *format_indent_more_icon[];
extern const char *left_arrow_icon[];
extern const char *log_out_icon[];
extern const char *minus_icon[];
extern const char *net_icon[];
extern const char *plus_icon[];
extern const char *process_stop_icon[];
extern const char *right_arrow_icon[];
extern const char *save_as_icon[];
extern const char *save_icon[];
extern const char *time_icon[];
extern const char *trash_icon[];
extern const char *image_icon[];
extern const char *weather_clear_icon[];
extern const char *start_here_icon[];
extern const char *help_browser_icon[];
extern const char *system_search_icon[];
extern const char *system_software_update_icon[];
extern const char *utilities_terminal_icon[];
extern const char *emblem_symbolic_link_icon[];
extern const char *emblems_system_icon[];
extern const char *dialog_information_icon[];
extern const char *executable_icon[];
extern const char *other_icon[];
extern const char *folder_open_icon[];
extern const char *preferences_system_icon[];
extern const char *preferences_desktop_font_icon[];
extern const char *system_users_icon[];
extern const char *utilities_system_monitor_icon[];
extern const char *multimedia_player_icon[];
extern const char *chat_icon[];
extern const char *dialog_information_48_icon[];
extern const char *dialog_warning_48_icon[];
extern const char *refresh_icon[];
extern const char *text_editor_icon[];
extern const char *text_icon[];
extern const char *shutdown_icon[];
extern const char *applications_system_icon[];
extern const char *audio_card_icon[];
extern const char *help_about_icon[];
extern const char *insert_link_icon[];
extern const char *close_icon[];
extern const char *enter_key_icon[];
extern const char *dialog_question_48_icon[];
extern const char *clear_sq_icon[];
extern const char *clear_row_icon[];
extern const char *rx1_icon[];
extern const char *tx1_icon[];
extern const char *rx2_icon[];
extern const char *tx2_icon[];
extern const char *rx_icon[];
extern const char *tx_icon[];
extern const char *fldigi_icon[];
extern const char *flarq_icon[];
extern const char *waterfall_icon[];
extern const char *dice_icon[];
extern const char *pskr_icon[];

#endif // PIXMAPS_H_
