// ----------------------------------------------------------------------------
// Copyright (C) 2014
//              David Freese, W1HKJ
//
// This file is part of flrig.
//
// flrig is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// flrig is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------

#ifndef IMAGES_H
#define IMAGES_H

#include <FL/Fl_Bitmap.H>

extern Fl_Bitmap image_p25;
extern Fl_Bitmap image_p50;
extern Fl_Bitmap image_p100;
extern Fl_Bitmap image_p200;
extern Fl_Bitmap image_p200log;

extern Fl_Bitmap image_smeter;
extern Fl_Bitmap image_swr;
extern Fl_Bitmap image_alc;

#endif
